/**************************************************************************************************
  Filename:       Multicast.c
  Revised:        $Date: 2009-03-18 15:56:27 -0700 (Wed, 18 Mar 2009) $
  Revision:       $Revision: 19453 $

  Description:    Sample Application (no Profile).


  Copyright 2007 Texas Instruments Incorporated. All rights reserved.

  IMPORTANT: Your use of this Software is limited to those specific rights
  granted under the terms of a software license agreement between the user
  who downloaded the software, his/her employer (which must be your employer)
  and Texas Instruments Incorporated (the "License").  You may not use this
  Software unless you agree to abide by the terms of the License. The License
  limits your use, and you acknowledge, that the Software may not be modified,
  copied or distributed unless embedded on a Texas Instruments microcontroller
  or used solely and exclusively in conjunction with a Texas Instruments radio
  frequency transceiver, which is integrated into your product.  Other than for
  the foregoing purpose, you may not use, reproduce, copy, prepare derivative
  works of, modify, distribute, perform, display or sell this Software and/or
  its documentation for any purpose.

  YOU FURTHER ACKNOWLEDGE AND AGREE THAT THE SOFTWARE AND DOCUMENTATION ARE
  PROVIDED AS IS?WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED,
  INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, TITLE,
  NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL
  TEXAS INSTRUMENTS OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER CONTRACT,
  NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR OTHER
  LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
  INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE
  OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT
  OF SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
  (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.

  Should you have any questions regarding your right to use this Software,
  contact Texas Instruments Incorporated at www.TI.com.
**************************************************************************************************/

/*********************************************************************
  This application isn't intended to do anything useful, it is
  intended to be a simple example of an application's structure.

  This application sends it's messages either as broadcast or
  broadcast filtered group messages.  The other (more normal)
  message addressing is unicast.  Most of the other sample
  applications are written to support the unicast message model.

  Key control:
    SW1:  Sends a flash command to all devices in Group 1.
    SW2:  Adds/Removes (toggles) this device in and out
          of Group 1.  This will enable and disable the
          reception of the flash command.
*********************************************************************/

/*********************************************************************
 * INCLUDES
 */
#include "OSAL.h"
#include "ZGlobals.h"
#include "AF.h"
#include "aps_groups.h"
#include "ZDApp.h"

#include "Multicast.h"
#include "MulticastHw.h"

#include "OnBoard.h"

/* HAL */
#include "hal_lcd.h"
#include "hal_led.h"
#include "hal_key.h"
#include "hal_uart.h" //for uart - liwen
#include "hal_adc.h" // read temperature -liwen
//#include  "hal_mcu.h" // read temperature -liwen

/*********************************************************************
 * MACROS
 */

/*********************************************************************
* CONSTANTS
 */


//for uart - liwen
// General UART frame offsets
#define FRAME_SOF_OFFSET                    0
#define FRAME_LENGTH_OFFSET                 1 
#define FRAME_CMD0_OFFSET                   2
#define FRAME_CMD1_OFFSET                   3
#define FRAME_DATA_OFFSET                   4

// ZB_RECEIVE_DATA_INDICATION offsets
#define ZB_RECV_SRC_OFFSET                  0
#define ZB_RECV_CMD_OFFSET                  2
#define ZB_RECV_LEN_OFFSET                  4
#define ZB_RECV_DATA_OFFSET                 6
#define ZB_RECV_FCS_OFFSET                  8

// ZB_RECEIVE_DATA_INDICATION frame length
#define ZB_RECV_LENGTH                      15

// PING response frame length and offset
#define SYS_PING_RSP_LENGTH                 7 
#define SYS_PING_CMD_OFFSET                 1

// Stack Profile
#define ZIGBEE_2007                         0x0040
#define ZIGBEE_PRO_2007                     0x0041

#ifdef ZIGBEEPRO
#define STACK_PROFILE                       ZIGBEE_PRO_2007             
#else 
#define STACK_PROFILE                       ZIGBEE_2007
#endif


#define CPT_SOP                             0xFE
#define SYS_PING_REQUEST                    0x0021
#define SYS_PING_RESPONSE                   0x0161
#define ZB_RECEIVE_DATA_INDICATION          0x8746

//SerialApp - liwen
// This is the max byte count per OTA message.
#if !defined( SERIAL_APP_TX_MAX )
#define SERIAL_APP_TX_MAX  80
#endif

// ADC definitions for CC2430/CC2530 from the hal_adc.c file
#if defined (HAL_MCU_CC2530)
#define HAL_ADC_REF_125V    0x00    /* Internal 1.25V Reference */
#define HAL_ADC_DEC_064     0x00    /* Decimate by 64 : 8-bit resolution */
#define HAL_ADC_DEC_128     0x10    /* Decimate by 128 : 10-bit resolution */
#define HAL_ADC_DEC_512     0x30    /* Decimate by 512 : 14-bit resolution */
#define HAL_ADC_CHN_VDD3    0x0f    /* Input channel: VDD/3 */
#define HAL_ADC_CHN_TEMP    0x0e    /* Temperature sensor */
#endif // HAL_MCU_CC2530


/*********************************************************************
 * TYPEDEFS
 */

//for uart - liwen
typedef struct
{
  uint16              source;
  uint16              parent;
  uint8               data;
//  uint8               temp;
//  uint8               voltage;
} gtwData_t;

/*********************************************************************
 * GLOBAL VARIABLES
 */

// This list should be filled with Application specific Cluster IDs.
const cId_t Multicast_ClusterList[MULTICAST_MAX_CLUSTERS] =
{
  MULTICAST_PERIODIC_CLUSTERID,
  MULTICAST_FLASH_CLUSTERID,
   MY_TEMP_CLUSTERID,
   MY_CMD_CLUSTERID
};

const SimpleDescriptionFormat_t Multicast_SimpleDesc =
{
  MULTICAST_ENDPOINT,              //  int Endpoint;
  MULTICAST_PROFID,                //  uint16 AppProfId[2];
  MULTICAST_DEVICEID,              //  uint16 AppDeviceId[2];
  MULTICAST_DEVICE_VERSION,        //  int   AppDevVer:4;
  MULTICAST_FLAGS,                 //  int   AppFlags:4;
  MULTICAST_MAX_CLUSTERS,          //  uint8  AppNumInClusters;
  (cId_t *)Multicast_ClusterList,  //  uint8 *pAppInClusterList;
  MULTICAST_MAX_CLUSTERS,          //  uint8  AppNumInClusters;
  (cId_t *)Multicast_ClusterList   //  uint8 *pAppInClusterList;
};

// This is the Endpoint/Interface description.  It is defined here, but
// filled-in in Multicast_Init().  Another way to go would be to fill
// in the structure here and make it a "const" (in code space).  The
// way it's defined in this sample app it is define in RAM.
endPointDesc_t Multicast_epDesc;

/*********************************************************************
 * EXTERNAL VARIABLES
 */

/*********************************************************************
 * EXTERNAL FUNCTIONS
 */

/*********************************************************************
 * LOCAL VARIABLES
 */
uint8 Multicast_TaskID;   // Task ID for internal task/event processing
                          // This variable will be received when
                          // Multicast_Init() is called.
devStates_t Multicast_NwkState;

uint8 Multicast_TransID;  // This is the unique message ID (counter)

afAddrType_t Multicast_Periodic_DstAddr;
afAddrType_t Multicast_Group_DstAddr;

aps_Group_t Multicast_Group;

uint8 MulticastPeriodicCounter = 0;
uint8 MulticastFlashCounter = 0;

//for uart - liwen
static gtwData_t gtwData;

//SerialApp - liwen
static uint8 SerialApp_TxSeq;
static uint8 SerialApp_TxBuf[SERIAL_APP_TX_MAX+1];
static uint8 SerialApp_TxLen;

/*********************************************************************
 * LOCAL FUNCTIONS
 */
void Multicast_HandleKeys( uint8 shift, uint8 keys );
void Multicast_MessageMSGCB( afIncomingMSGPacket_t *pckt );
void Multicast_SendPeriodicMessage( void );
void Multicast_SendFlashMessage( uint16 flashTime );
void sendTempMessage(void);
static int8 readTemp(void);

//for uart - liwen
//static uint8 calcFCS(uint8 *pBuf, uint8 len);
//static void sysPingReqRcvd(void);
//static void sysPingRsp(void);

void zb_ReceiveDataIndication( uint16 source, uint16 command, uint16 len, uint8 *pData  );
//void zb_ReceiveDataIndication( uint8 *pData  );
static void sendGtwReport(gtwData_t *gtwData);
//static void sendGtwReport(uint8 *gtwData);

static void SerialApp_CallBack(uint8 port, uint8 event);
void SendToEndDevice(void); // similar to Multicast_SendFlashMessage()

//read temperature -liwen



/*********************************************************************
 * NETWORK LAYER CALLBACKS
 */

/*********************************************************************
 * PUBLIC FUNCTIONS
 */

/*********************************************************************
 * @fn      Multicast_Init
 *
 * @brief   Initialization function for the Generic App Task.
 *          This is called during initialization and should contain
 *          any application specific initialization (ie. hardware
 *          initialization/setup, table initialization, power up
 *          notificaiton ... ).
 *
 * @param   task_id - the ID assigned by OSAL.  This ID should be
 *                    used to send messages and set timers.
 *
 * @return  none
 */
void Multicast_Init( uint8 task_id )
{
  Multicast_TaskID = task_id;
  Multicast_NwkState = DEV_INIT;
  Multicast_TransID = 0;

  // Device hardware initialization can be added here or in main() (Zmain.c).
  // If the hardware is application specific - add it here.
  // If the hardware is other parts of the device add it in main().

  // Initialise UART - liwen
  //initUart(uartRxCB);
 
  //SerialApp - liwen
  MyInitUart();
  SerialApp_TxLen = 0;
  
 #if defined ( BUILD_ALL_DEVICES )
  // The "Demo" target is setup to have BUILD_ALL_DEVICES and HOLD_AUTO_START
  // We are looking at a jumper (defined in MulticastHw.c) to be jumpered
  // together - if they are - we will start up a coordinator. Otherwise,
  // the device will start as a router.
  if ( readCoordinatorJumper() )
    zgDeviceLogicalType = ZG_DEVICETYPE_COORDINATOR;
  else
    zgDeviceLogicalType = ZG_DEVICETYPE_ROUTER;
#endif // BUILD_ALL_DEVICES

#if defined ( HOLD_AUTO_START )
  // HOLD_AUTO_START is a compile option that will surpress ZDApp
  //  from starting the device and wait for the application to
  //  start the device.
  ZDOInitDevice(0);
#endif

  // Setup for the periodic message's destination address
  // Broadcast to everyone
  Multicast_Periodic_DstAddr.addrMode = (afAddrMode_t)AddrBroadcast;
  Multicast_Periodic_DstAddr.endPoint = MULTICAST_ENDPOINT;
  Multicast_Periodic_DstAddr.addr.shortAddr = 0xFFFF;

  // Setup for the flash command's destination address - Group 1
  Multicast_Group_DstAddr.addrMode = (afAddrMode_t)afAddrGroup;
  Multicast_Group_DstAddr.endPoint = MULTICAST_ENDPOINT;
  Multicast_Group_DstAddr.addr.shortAddr = MULTICAST_TRANSLATOR_GROUP;

  // Fill out the endpoint description.
  Multicast_epDesc.endPoint = MULTICAST_ENDPOINT;
  Multicast_epDesc.task_id = &Multicast_TaskID;
  Multicast_epDesc.simpleDesc
            = (SimpleDescriptionFormat_t *)&Multicast_SimpleDesc;
  Multicast_epDesc.latencyReq = noLatencyReqs;

  // Register the endpoint description with the AF
  afRegister( &Multicast_epDesc );

  // Register for all key events - This app will handle all key events
  RegisterForKeys( Multicast_TaskID );

  // By default, all devices start out in Group 1
  Multicast_Group.ID = 0x0001;
  osal_memcpy( Multicast_Group.name, "Group 1", 7  );
  aps_AddGroup( MULTICAST_ENDPOINT, &Multicast_Group );
 
}

/******************************************************************************
 * @fn          MyInitUart - liwen
 *
 * @brief       Initialise UART 
 *
 * @param       pf - Function pointer to UART callback function
 *              
 * @return      none
 */
void MyInitUart(void)
{
  halUARTCfg_t uartConfig;
  
  uartConfig.configured           = TRUE;              
  uartConfig.baudRate             = HAL_UART_BR_38400;
  uartConfig.flowControl          = FALSE;
  uartConfig.flowControlThreshold = 48;
  uartConfig.rx.maxBufSize        = RX_BUF_LEN;
  uartConfig.tx.maxBufSize        = 128;
  
  uartConfig.idleTimeout          = 6;   
  uartConfig.intEnable            = TRUE;              
  uartConfig.callBackFunc         = SerialApp_CallBack;
  
  HalUARTOpen (HAL_UART_PORT_0, &uartConfig);
}

/*********************************************************************
 * @fn      Multicast_ProcessEvent
 *
 * @brief   Generic Application Task event processor.  This function
 *          is called to process all events for the task.  Events
 *          include timers, messages and any other user defined events.
 *
 * @param   task_id  - The OSAL assigned task ID.
 * @param   events - events to process.  This is a bit map and can
 *                   contain more than one event.
 *
 * @return  none
 */
uint16 Multicast_ProcessEvent( uint8 task_id, uint16 events )
{
  afIncomingMSGPacket_t *MSGpkt;
  (void)task_id;  // Intentionally unreferenced parameter

  if ( events & SYS_EVENT_MSG )
  {
    MSGpkt = (afIncomingMSGPacket_t *)osal_msg_receive( Multicast_TaskID );
    while ( MSGpkt )
    {
      switch ( MSGpkt->hdr.event )
      {
        // Received when a key is pressed
        case KEY_CHANGE:
          Multicast_HandleKeys( ((keyChange_t *)MSGpkt)->state, ((keyChange_t *)MSGpkt)->keys );
          break;

        // Received when a messages is received (OTA) for this endpoint
        case AF_INCOMING_MSG_CMD:
          Multicast_MessageMSGCB( MSGpkt );
          break;

        // Received whenever the device changes state in the network
        case ZDO_STATE_CHANGE:
      /*    Multicast_NwkState = (devStates_t)(MSGpkt->hdr.status);
          if ( (Multicast_NwkState == DEV_ZB_COORD)
              || (Multicast_NwkState == DEV_ROUTER)
              || (Multicast_NwkState == DEV_END_DEVICE) )
          {
            // Start sending the periodic message in a regular interval.
            osal_start_timerEx( Multicast_TaskID,
                              MULTICAST_SEND_PERIODIC_MSG_EVT,
                              MULTICAST_SEND_PERIODIC_MSG_TIMEOUT );
          }
          else
          {
            // Device is no longer in the network
          }*/
          break;

        default:
          break;
      }

      // Release the memory
      osal_msg_deallocate( (uint8 *)MSGpkt );

      // Next - if one is available
      MSGpkt = (afIncomingMSGPacket_t *)osal_msg_receive( Multicast_TaskID );
    }

    // return unprocessed events
    return (events ^ SYS_EVENT_MSG);
  }

  // Send a message out - This event is generated by a timer
  //  (setup in Multicast_Init()).
  /*if ( events & MULTICAST_SEND_PERIODIC_MSG_EVT )
  {
    // Send the periodic message
    Multicast_SendPeriodicMessage();

    // Setup to send message again in normal period (+ a little jitter)
    osal_start_timerEx( Multicast_TaskID, MULTICAST_SEND_PERIODIC_MSG_EVT,
        (MULTICAST_SEND_PERIODIC_MSG_TIMEOUT + (osal_rand() & 0x00FF)) );

    // return unprocessed events
    return (events ^ MULTICAST_SEND_PERIODIC_MSG_EVT);
  }*/

  // Discard unknown events
  return 0;
}

/*********************************************************************
 * Event Generation Functions
 */
/*********************************************************************
 * @fn      Multicast_HandleKeys
 *
 * @brief   Handles all key events for this device.
 *
 * @param   shift - true if in shift/alt.
 * @param   keys - bit field for key events. Valid entries:
 *                 HAL_KEY_SW_2
 *                 HAL_KEY_SW_1
 *
 * @return  none
 */
void Multicast_HandleKeys( uint8 shift, uint8 keys )
{
  (void)shift;  // Intentionally unreferenced parameter
  
  if ( keys & HAL_KEY_SW_1 )
  {
    /* This key sends the Flash Command is sent to Group 1.
     * This device will not receive the Flash Command from this
     * device (even if it belongs to group 1).
     */
    Multicast_SendFlashMessage( MULTICAST_FLASH_DURATION );
  }

  if ( keys & HAL_KEY_SW_2 )
  {
    /* The Flashr Command is sent to Group 1.
     * This key toggles this device in and out of group 1.
     * If this device doesn't belong to group 1, this application
     * will not receive the Flash command sent to group 1.
     */
    
    /*only gateways (router) join Group 1 - liwen */
    aps_Group_t *grp;
    grp = aps_FindGroup( MULTICAST_ENDPOINT, MULTICAST_TRANSLATOR_GROUP );
    if ( grp )
    {
      // Remove from the group
      aps_RemoveGroup( MULTICAST_ENDPOINT, MULTICAST_TRANSLATOR_GROUP );
      #if defined ( LCD_SUPPORTED )
         HalLcdWriteString( "RM from group", HAL_LCD_LINE_2 );
      #endif
      //HalLedSet ( HAL_LED_4, HAL_LED_MODE_BLINK );
    }
    else
    {
      // Add to the flash group
      aps_AddGroup( MULTICAST_ENDPOINT, &Multicast_Group );
      #if defined ( LCD_SUPPORTED )
         HalLcdWriteString( "ADD to group", HAL_LCD_LINE_2 );
      #endif
      //HalLedSet ( HAL_LED_4, HAL_LED_MODE_BLINK );
    }
  }
  if ( keys & HAL_KEY_SW_3 )
  {
    sendTempMessage();
  }
}

/*********************************************************************
 * LOCAL FUNCTIONS
 */

/*********************************************************************
 * @fn      Multicast_MessageMSGCB
 *
 * @brief   Data message processor callback.  This function processes
 *          any incoming data - probably from other devices.  So, based
 *          on cluster ID, perform the intended action.
 *
 * @param   none
 *
 * @return  none
 */
void Multicast_MessageMSGCB( afIncomingMSGPacket_t *pkt )
{
  uint16 flashTime;

  switch ( pkt->clusterId )
  {
  case MULTICAST_PERIODIC_CLUSTERID:
    flashTime = BUILD_UINT16(LO_UINT16( MULTICAST_FLASH_DURATION ), 
                             HI_UINT16( MULTICAST_FLASH_DURATION ) );
    HalLedBlink( HAL_LED_4, 4, 50, (flashTime / 4) );  
    zb_ReceiveDataIndication( pkt->srcAddr.addr.shortAddr, pkt->clusterId,
                          pkt->cmd.DataLength, pkt->cmd.Data);
    break;
  case MULTICAST_FLASH_CLUSTERID:
    flashTime = BUILD_UINT16(pkt->cmd.Data[1], pkt->cmd.Data[2] );
    HalLedBlink( HAL_LED_4, 4, 50, (flashTime / 4) );     
    zb_ReceiveDataIndication( pkt->srcAddr.addr.shortAddr, pkt->clusterId,
                                  pkt->cmd.DataLength, pkt->cmd.Data);
    break;
  case MY_TEMP_CLUSTERID:
    flashTime = BUILD_UINT16(LO_UINT16( MULTICAST_FLASH_DURATION ), 
                             HI_UINT16( MULTICAST_FLASH_DURATION ) );
    HalLedBlink( HAL_LED_4, 4, 50, (flashTime / 4) );  
    zb_ReceiveDataIndication( pkt->srcAddr.addr.shortAddr, pkt->clusterId,
                          pkt->cmd.DataLength, pkt->cmd.Data);
    break;
  case MY_CMD_CLUSTERID:
    HalLedSet ( HAL_LED_1, HAL_LED_MODE_BLINK );
    break;
  }
}

/*********************************************************************
 * @fn      Multicast_SendPeriodicMessage
 *
 * @brief   Send the periodic message.
 *
 * @param   none
 *
 * @return  none
 */
void Multicast_SendPeriodicMessage( void )
{
  if ( AF_DataRequest( &Multicast_Periodic_DstAddr, &Multicast_epDesc,
                       MULTICAST_PERIODIC_CLUSTERID,
                       1,
                       (uint8*)&MulticastPeriodicCounter,
                       &Multicast_TransID,
                       AF_DISCV_ROUTE,
                       AF_DEFAULT_RADIUS ) == afStatus_SUCCESS )
  {
  }
  else
  {
    // Error occurred in request to send.
  }
}

/*********************************************************************
 * @fn      Multicast_SendFlashMessage
 *
 * @brief   Send the flash message to group 1.
 *
 * @param   flashTime - in milliseconds
 *
 * @return  none
 */
void Multicast_SendFlashMessage( uint16 flashTime )
{
  uint8 buffer[3];
  buffer[0] = (uint8)(MulticastFlashCounter++);
  buffer[1] = LO_UINT16( flashTime );
  buffer[2] = HI_UINT16( flashTime );

  if ( AF_DataRequest( &Multicast_Group_DstAddr, &Multicast_epDesc,
                       MULTICAST_FLASH_CLUSTERID,
                       3,
                       buffer,
                       &Multicast_TransID,
                       AF_DISCV_ROUTE,
                       AF_DEFAULT_RADIUS ) == afStatus_SUCCESS )
  {
  }
  else
  {
    // Error occurred in request to send.
  }
}

/******************************************************************************
 * @fn          sendTempMessage
 *
 * @brief       Send sensor report
 *
 * @param       none
 *              
 * @return      none
 */
void sendTempMessage(void)
{
  uint8 pData[1];
  pData[0] =  readTemp();
  
  if ( AF_DataRequest( &Multicast_Group_DstAddr, &Multicast_epDesc,
                         MY_TEMP_CLUSTERID,
                         1,
                         pData,
                         &Multicast_TransID,
                         AF_DISCV_ROUTE,
                         AF_DEFAULT_RADIUS )== afStatus_SUCCESS )
  {
  }
  else
  {
    // Error occurred in request to send.
  }
}

/******************************************************************************
 * @fn          readTemp
 *
 * @brief       read temperature from ADC
 *
 * @param       none
 *              
 * @return      temperature
 */
static int8 readTemp(void)
{
  static uint16 voltageAtTemp22;
  static uint8 bCalibrate=TRUE; // Calibrate the first time the temp sensor is read
  uint16 value;
  int8 temp=100;

  #if defined (HAL_MCU_CC2530)
  ATEST = 0x01;
  TR0  |= 0x01; 
  
  /* Clear ADC interrupt flag */
  ADCIF = 0;

  ADCCON3 = (HAL_ADC_REF_125V | HAL_ADC_DEC_512 | HAL_ADC_CHN_TEMP);

  /* Wait for the conversion to finish */
  while ( !ADCIF );

  /* Get the result */
  value = ADCL;
  value |= ((uint16) ADCH) << 8;

  // Use the 12 MSB of adcValue
  value >>= 4;
  
  /*
   * These parameters are typical values and need to be calibrated
   * See the datasheet for the appropriate chip for more details
   * also, the math below may not be very accurate
   */
    /* Assume ADC = 1480 at 25C and ADC = 4/C */
  #define VOLTAGE_AT_TEMP_25        1480
  #define TEMP_COEFFICIENT          4

  // Calibrate for 22C the first time the temp sensor is read.
  // This will assume that the demo is started up in temperature of 22C
  if(bCalibrate) {
    voltageAtTemp22=value;
    bCalibrate=FALSE;
  }
  
  temp = 22 + ( (value - voltageAtTemp22) / TEMP_COEFFICIENT );
  
  // Set 0C as minimum temperature, and 100C as max
  if( temp >= 100) 
  {
    return 100;
  }
  else if (temp <= 0) {
    return 0;
  }
  else { 
    return temp;
  }
  // Only CC2530 is supported
  #else
  return 0;
  #endif
}

/******************************************************************************
 * @fn          zb_ReceiveDataIndication - liwen
 *
 * @brief       The zb_ReceiveDataIndication callback function is called
 *              asynchronously by the ZigBee stack to notify the application
 *              when data is received from a peer device.
 *
 * @param       source - The short address of the peer device that sent the data
 *              command - The commandId associated with the data
 *              len - The number of bytes in the pData parameter
 *              pData - The data sent by the peer device
 *
 * @return      none
 */
void zb_ReceiveDataIndication( uint16 source, uint16 clusterId, uint16 len, uint8 *pData  )
{ 
  gtwData.source=source;
  gtwData.parent = BUILD_UINT16(pData[SENSOR_PARENT_OFFSET+ 1], pData[SENSOR_PARENT_OFFSET]);  
  gtwData.data=*pData;
//  gtwData.voltage=*(pData+1);
  
  //HalLedSet ( HAL_LED_2, HAL_LED_MODE_FLASH );
  
  // Update the display
  //#if defined ( LCD_SUPPORTED )
  //HalLcdWriteScreen( "Report", "rcvd" );
  //#endif
  
  // Send gateway report
  sendGtwReport(&gtwData);
  //sendGtwReport(pData);
}

/******************************************************************************
 * @fn          sendGtwReport - liwen
 *
 * @brief       Build and send gateway report
 *
 * @param       none
 *              
 * @return      none
 */
static void sendGtwReport(gtwData_t *gtwData)
{
  uint8 pFrame[5];
  pFrame[0]=LO_UINT16(gtwData->source); 
  pFrame[1]=HI_UINT16(gtwData->source); 
  pFrame[2]=LO_UINT16(gtwData->parent); 
  pFrame[3]=HI_UINT16(gtwData->parent);
  pFrame[4]=gtwData->data;
  HalUARTWrite(HAL_UART_PORT_0,pFrame, 5);
}

/*********************************************************************
*********************************************************************/


/*********************************************************************
* @fn      SerialApp_CallBack - liwen: for read UART 
 *
 * @brief   Send data OTA.
 *
 * @param   port - UART port.
 * @param   event - the UART port event flag.
 *
 * @return  none
 */
static void SerialApp_CallBack(uint8 port, uint8 event)
{
  (void)port;

  if ((event & (HAL_UART_RX_FULL | HAL_UART_RX_ABOUT_FULL | HAL_UART_RX_TIMEOUT)) &&
      !SerialApp_TxLen)
  {
    SendToEndDevice();
  }
}
/*********************************************************************
* @fn      SendToEndDevice - liwen: read UART and send to ZigBee device
 *
 * @brief   Send data OTA.
 *
 * @param   port - UART port.
 * @param   event - the UART port event flag.
 *
 * @return  none
 */
void SendToEndDevice()
{
  afAddrType_t MyBroadcastAddr;
  MyBroadcastAddr.addrMode = (afAddrMode_t)AddrBroadcast;
  MyBroadcastAddr.endPoint = MULTICAST_ENDPOINT;
  MyBroadcastAddr.addr.shortAddr = 0xFFFF;
  
  if (!SerialApp_TxLen && 
      (SerialApp_TxLen = HalUARTRead(HAL_UART_PORT_0, SerialApp_TxBuf+1, SERIAL_APP_TX_MAX)))
  {
    // Pre-pend sequence number to the Tx message.
    SerialApp_TxBuf[0] = ++SerialApp_TxSeq;
  }

  if (SerialApp_TxLen)
  {
    if ( AF_DataRequest( &MyBroadcastAddr, &Multicast_epDesc,
                         MY_CMD_CLUSTERID,
                         SerialApp_TxLen+1,
                         SerialApp_TxBuf,
                         &Multicast_TransID,
                         AF_DISCV_ROUTE,
                         AF_DEFAULT_RADIUS ) == afStatus_SUCCESS )
    {
       SerialApp_TxLen=0;
    }
    else
    {
      // Error occurred in request to send.
    }
  }
}

/*********************************************************************
*********************************************************************/